'''Any building operations'''
import bge
import logging

import Map
import UI
import Units
import Economy

LOG = logging.getLogger()


def get_entry():
    '''Returns the entry'''
    main_scene = bge.logic.getCurrentScene()
    return [o for o in main_scene.objects if isinstance(o, EntryBuilding)][0]


def place_entry(pos):
    '''Places an entry object at the specified location'''
    scene = bge.logic.getCurrentScene()
    entry = scene.addObject('EntryConcretePad', scene.objects[0])
    EntryBuilding(entry, pos)


class EntryBuilding(bge.types.KX_GameObject):
    '''Makes an entry building from a given object'''
    def __init__(self, _, pos):
        LOG.debug("Made Entry")
        self.pos = pos
        self.worldScale = [1, 1, 1]
        self.worldPosition = pos
        self.worldPosition.x -= 0.5
        self.worldPosition.y -= 0.5
        self.worldPosition.z = -0.1
        self.worldOrientation = [0, 0, 0]
        self['Droppable'] = True

        node_x, node_y = Map.get_closest_node(self.worldPosition)
        bge.logic.map.add_placable(node_x, node_y, self)
        bge.logic.map.add_placable(node_x + 1, node_y, self)
        bge.logic.map.add_placable(node_x, node_y + 1, self)
        bge.logic.map.add_placable(node_x + 1, node_y + 1, self)
        self.map_pos = [node_x, node_y]

        UI.get_main_display().set_position(self.worldPosition.xy)
        self.play_animation(-1)

    def get_map_position(self):
        return self.map_pos

    def play_animation(self, direction):
        '''Animates the elevator going up and down'''
        if direction == 1:
            start = 0
            stop = 100
            speed = 1
        elif direction == -1:
            start = 100
            stop = 0
            speed = 1
        for child in self.childrenRecursive:
            if 'Animate' in child:
                action_name = child.name
                child.playAction(action_name, start, stop, speed=speed)

    def spawn_object(self, obj_data, cost):
        '''Spawns an object at the mine entry'''
        if Economy.get_wallet().withdraw(cost):
            spawn_pos = self.children['EntrySpawnPos']
            Units.add_unit(obj_data, spawn_pos)

    def add_resources(self, resource_dict):
        Economy.get_wallet().add_resource(resource_dict)
