'''Keeps track of what the mouse is doing when over the main display'''
import math

import bge

import defs
import UI
import Map
import Economy
import Control
import Units

selected = None


def set_selected(thing):
    '''Sets the selected objects'''
    global selected
    selected = thing


def get_selected():
    '''Returns the selected object'''
    return selected


def handle(event):
    '''Happens when there is a non-ui event on the main screen'''
    if Control.get_key('ESCKEY') == 1:
        if get_selected() is not None:
            do_selected(None, None, selected, None)
            set_selected(None)
            Units.deselect_all_units()
        else:
            bge.logic.endGame()

    if event is None:
        do_selected(None, None, selected, None)

    else:
        hover, hitpos = get_mouse_over(event)

        if hover is not None:
            pos = Map.get_closest_node(hitpos, inter=0)
            do_selected(event, pos, selected, hover)


def do_selected(event, pos, selec, hover):
    '''Forks the data to whatever is selected'''
    if selec is None:
        # mining(event, pos)
        Units.select_unit(event, pos, selec, hover)
    else:
        if isinstance(selec, UI.BuildingButton):
            building(event, pos, selec, hover)
            Units.deselect_all_units()
        if selec == Units.assign_target:
            Units.assign_target(event, pos, selec, hover)


def get_mouse_over(event):
    '''Returns the object the mouse is over'''
    main_display = UI.get_main_display()

    cam_pos = event['uv']
    vect = main_display.cam.getScreenVect(cam_pos[0], cam_pos[1])
    ray_to = main_display.cam.worldPosition - vect
    res = main_display.cam.rayCast(ray_to, main_display.cam, defs.WORLD_SIZE)
    hover, hitpos, _ = res
    return hover, hitpos


def mining(event, pos):
    '''Removed the map under the mouse'''
    if event is None:
        return
    if event[defs.MOUSE_MAP['SELECT']] != 0:
        bge.logic.map.mine(math.ceil(pos[0]), math.ceil(pos[1]), 1, True)


def building(event, pos, sel, _):
    '''Places a building at the specified location'''
    phantom = building.__dict__.get('phantom', None)

    if event is None:
        # Mouse moved off screen or no longer active selected
        if phantom is not None:
            phantom.endObject()
            building.phantom = None
        return
    pos[0] = math.ceil(pos[0])
    pos[1] = math.ceil(pos[1])
    terrain = bge.logic.map.get_point(pos[0], pos[1])
    placables = bge.logic.map.get_placable(pos[0], pos[1])
    if terrain > 0 or placables != 0:
        # Terrain already occupied or not suitable for building
        if phantom is not None:
            phantom.endObject()
            building.phantom = None
        return

    if phantom is None:
        # Add the phantom
        scene = bge.logic.getCurrentScene()
        ref = scene.objects[0]
        obj = scene.objectsInactive[sel.data['name']]
        building.phantom = scene.addObject(obj, ref)
        building.phantom.worldPosition = Map.node_to_world(pos)
        phantom = building.phantom
    elif phantom.name == sel.data['name']:
        phantom.worldPosition = Map.node_to_world(pos)

    if event['LEFTMOUSE'] == bge.logic.KX_SENSOR_JUST_ACTIVATED:
        # Make the phantom object real
        cost = {'Ore': sel.data['Ore'],
                'Crystal': sel.data['Crystal']}
        if Economy.get_wallet().withdraw(cost):  # Check cost
            bge.logic.map.add_placable(pos[0], pos[1], building.phantom)
            building.phantom = None
            if Control.get_key('LEFTSHIFTKEY') == 0:
                set_selected(None)

    if event['RIGHTMOUSE'] == bge.logic.KX_SENSOR_JUST_ACTIVATED:
        # Cancel with right mouse click
        set_selected(None)
        phantom.endObject()
        building.phantom = None
